/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/* $Id: AliTRDCalDCSPTRCba.cxx 18952 2007-06-08 11:36:12Z cblume $ */

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  TRD calibration class for TRD DCS GTU parameters                         //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#include "AliTRDCalDCSPTRCba.h"

ClassImp(AliTRDCalDCSPTRCba)

//_____________________________________________________________________________
AliTRDCalDCSPTRCba::AliTRDCalDCSPTRCba()
  :TNamed()
    ,fSide("")
    ,fPrimary(0)
    ,fPreToNextCycles(0)
    ,fL0ToNextCycles(0)
    ,fL1ToNextCycles(0)
    ,fClkTtcexShifts(0)
    ,fDisableGtuBusy(0)
    ,fDisableSorEorBusy(0)
    ,fScintEn(0)
    ,fLutAsPre(0)
    ,fCtpAsPre(0)
    ,fL0En(0)
    ,fL1En(0)
    ,fLutL0ToCtpEn(0)
    ,fLutPreToCtpEn(0)
    ,fSoftwTrigToCtpEn(0)
    ,fL0Autogenerate(0)
    ,fL1Autogenerate(0)
    ,fChannelBDisable(0)
    ,fTtcexClkDisable(0)
    ,fPreAsL0En(0)
    ,fNoTriggerToSm(0)
    ,fSoftwTrigEn(0)
    ,fSoftwTrigPattern(0)
    ,fSychrInput(0)
    ,fRandomTriggerThr(0)
    ,fPtChDelayCbA(0)
    ,fPtChDelayCbC(0)
    ,fPtChDelayGtu(0)
    ,fPtChDelayT0(0)
    ,fPtChDelayT0(0)
    ,fPtChDelayT1(0)
    ,fPtChDelayT2(0)
    ,fPtChDelayT3(0)
    ,fPtChDelayT4(0)
    ,fPtChDelayT5(0)
    ,fPtChDelayT6(0)
    ,fPtChDelayT7(0)
    ,fClkLb(0)
    ,fClkHb(0)
    ,fPulseToSmLb(0)
    ,fPulseToSmHb(0)
    ,fDeadTimeLb(0)
    ,fDeadTimeHb(0)
    ,fPrePulsesLb(0)
    ,fPrePulsesHb(0)
    ,fL0CtpLb(0)
    ,fL0CtpHb(0)
    ,fL1CtpLb(0)
    ,fL1CtpHb(0)
    ,fL0LutLb(0)
    ,fL0LutHb(0)
    ,fNotTinToCtpPreLb(0)
    ,fNotTinToCtpPreHb(0)
    ,fPatternMatchCbALb(0)
    ,fPatternMatchCbAHb(0)
    ,fPatternMatchCbCLb(0)
    ,fPatternMatchCbCHb(0)
    ,fPatternMatchCbTlmuLb(0)
    ,fPatternMatchCbTlmuHb(0)
    ,fTriggerSynchrLb(0)
    ,fTriggerSynchrHb(0)
    ,fTriggerToSmLb(0)
    ,fTriggerToSmHb(0)
    ,fPreLutLb(0)
    ,fPreLutHb(0)
    ,fMissingPreLb(0)
    ,fMissingPreHb(0)
    ,fUnnecessaryPreLb(0)
    ,fUnnecessaryPreHb(0)
    
{
  //
  // AliTRDCalDCSGTU default constructor
  //
  fSegmentsArr->SetOwner();
}

//_____________________________________________________________________________
AliTRDCalDCSPTRCba::AliTRDCalDCSPTRCba(const char *name, const char *title)
  :TNamed(name,title)
    ,fSide("")
    ,fPrimary(0)
    ,fPreToNextCycles(0)
    ,fL0ToNextCycles(0)
    ,fL1ToNextCycles(0)
    ,fClkTtcexShifts(0)
    ,fDisableGtuBusy(0)
    ,fDisableSorEorBusy(0)
    ,fScintEn(0)
    ,fLutAsPre(0)
    ,fCtpAsPre(0)
    ,fL0En(0)
    ,fL1En(0)
    ,fLutL0ToCtpEn(0)
    ,fLutPreToCtpEn(0)
    ,fSoftwTrigToCtpEn(0)
    ,fL0Autogenerate(0)
    ,fL1Autogenerate(0)
    ,fChannelBDisable(0)
    ,fTtcexClkDisable(0)
    ,fPreAsL0En(0)
    ,fNoTriggerToSm(0)
    ,fSoftwTrigEn(0)
    ,fSoftwTrigPattern(0)
    ,fSychrInput(0)
    ,fRandomTriggerThr(0)
    ,fPtChDelayCbA(0)
    ,fPtChDelayCbC(0)
    ,fPtChDelayGtu(0)
    ,fPtChDelayT0(0)
    ,fPtChDelayT0(0)
    ,fPtChDelayT1(0)
    ,fPtChDelayT2(0)
    ,fPtChDelayT3(0)
    ,fPtChDelayT4(0)
    ,fPtChDelayT5(0)
    ,fPtChDelayT6(0)
    ,fPtChDelayT7(0)
    ,fClkLb(0)
    ,fClkHb(0)
    ,fPulseToSmLb(0)
    ,fPulseToSmHb(0)
    ,fDeadTimeLb(0)
    ,fDeadTimeHb(0)
    ,fPrePulsesLb(0)
    ,fPrePulsesHb(0)
    ,fL0CtpLb(0)
    ,fL0CtpHb(0)
    ,fL1CtpLb(0)
    ,fL1CtpHb(0)
    ,fL0LutLb(0)
    ,fL0LutHb(0)
    ,fNotTinToCtpPreLb(0)
    ,fNotTinToCtpPreHb(0)
    ,fPatternMatchCbALb(0)
    ,fPatternMatchCbAHb(0)
    ,fPatternMatchCbCLb(0)
    ,fPatternMatchCbCHb(0)
    ,fPatternMatchCbTlmuLb(0)
    ,fPatternMatchCbTlmuHb(0)
    ,fTriggerSynchrLb(0)
    ,fTriggerSynchrHb(0)
    ,fTriggerToSmLb(0)
    ,fTriggerToSmHb(0)
    ,fPreLutLb(0)
    ,fPreLutHb(0)
    ,fMissingPreLb(0)
    ,fMissingPreHb(0)
    ,fUnnecessaryPreLb(0)
    ,fUnnecessaryPreHb(0)
{
  //
  // AliTRDCalDCSGTU constructor
  //
}

//_____________________________________________________________________________
AliTRDCalDCSPTRCba::AliTRDCalDCSPTRCba(const AliTRDCalDCSPTRCba &)
  :TNamed("","")
    ,fSide("")
    ,fPrimary(0)
    ,fPreToNextCycles(0)
    ,fL0ToNextCycles(0)
    ,fL1ToNextCycles(0)
    ,fClkTtcexShifts(0)
    ,fDisableGtuBusy(0)
    ,fDisableSorEorBusy(0)
    ,fScintEn(0)
    ,fLutAsPre(0)
    ,fCtpAsPre(0)
    ,fL0En(0)
    ,fL1En(0)
    ,fLutL0ToCtpEn(0)
    ,fLutPreToCtpEn(0)
    ,fSoftwTrigToCtpEn(0)
    ,fL0Autogenerate(0)
    ,fL1Autogenerate(0)
    ,fChannelBDisable(0)
    ,fTtcexClkDisable(0)
    ,fPreAsL0En(0)
    ,fNoTriggerToSm(0)
    ,fSoftwTrigEn(0)
    ,fSoftwTrigPattern(0)
    ,fSychrInput(0)
    ,fRandomTriggerThr(0)
    ,fPtChDelayCbA(0)
    ,fPtChDelayCbC(0)
    ,fPtChDelayGtu(0)
    ,fPtChDelayT0(0)
    ,fPtChDelayT0(0)
    ,fPtChDelayT1(0)
    ,fPtChDelayT2(0)
    ,fPtChDelayT3(0)
    ,fPtChDelayT4(0)
    ,fPtChDelayT5(0)
    ,fPtChDelayT6(0)
    ,fPtChDelayT7(0)
    ,fClkLb(0)
    ,fClkHb(0)
    ,fPulseToSmLb(0)
    ,fPulseToSmHb(0)
    ,fDeadTimeLb(0)
    ,fDeadTimeHb(0)
    ,fPrePulsesLb(0)
    ,fPrePulsesHb(0)
    ,fL0CtpLb(0)
    ,fL0CtpHb(0)
    ,fL1CtpLb(0)
    ,fL1CtpHb(0)
    ,fL0LutLb(0)
    ,fL0LutHb(0)
    ,fNotTinToCtpPreLb(0)
    ,fNotTinToCtpPreHb(0)
    ,fPatternMatchCbALb(0)
    ,fPatternMatchCbAHb(0)
    ,fPatternMatchCbCLb(0)
    ,fPatternMatchCbCHb(0)
    ,fPatternMatchCbTlmuLb(0)
    ,fPatternMatchCbTlmuHb(0)
    ,fTriggerSynchrLb(0)
    ,fTriggerSynchrHb(0)
    ,fTriggerToSmLb(0)
    ,fTriggerToSmHb(0)
    ,fPreLutLb(0)
    ,fPreLutHb(0)
    ,fMissingPreLb(0)
    ,fMissingPreHb(0)
    ,fUnnecessaryPreLb(0)
    ,fUnnecessaryPreHb(0)
    
{
  //
  // AliTRDCalDCSGTU constructor
  //
}



